package net.sf.sql2java.common.parameters;

import java.util.Map;

import net.sf.sql2java.common.parameters.handlers.ContextParameterHandler;

/**
 * An object holding data in a momentum fashion.
 * 
 * A momentum is an interval during some period. In the case of a 
 * MomentumObject the period will be the object's lifecycle and the momentum
 * defines a certain moment in time. 
 * <p>
 * The values stored inside a momentum object are specific for a certain moment
 * in time. Despite the current state, an original state is hold so values of a
 * certain state may be matched to the original ones.
 * <p>
 * A MomentumObject will be used to create objects which change depending on
 * the context they are used in. Most of the time, the original state will hold
 * some sort of formula to be parsed depending on the context.
 * <p>
 * Our implementation will use a <code>StringParser</code> to parse the 
 * original values and generate a certain state.
 * 
 * @author Daan Gerits
 */
public abstract class MomentumObject {
	private String internalName;
	
	private MomentumObjectState state;
	private MomentumObjectState originalState;

// -- Constructors ------------------------------------------------------------
	
	/**
	 * The default constructor for a momentum object.
	 * <p>
	 * The internal name for the object needs to be provided so we can resolve
	 * any references to it when parsing formulas. For example, if the internal
	 * name would be 'template' and the formula 'template.name', the parser 
	 * will replace the formula with the name property of this object.
	 * 
	 * @param internalname	The internal name to be used
	 */
	public MomentumObject(String internalname) {
		this.internalName = internalname;
		
		// -- create the original state
		originalState = new MomentumObjectState();
		
		// -- set the current state as the original
		state = originalState;
	}
	
// -- Parameter registration --------------------------------------------------

	/**
	 * Set a property formula.
	 * 
	 * Setting a property means we will set the value of the property inside
	 * the original state. It won't be right to modify it inside the current
	 * state since the current state will not always match the original. 
	 * <p>
	 * When we parse this object, the current state will point to the state
	 * holding all parsed values.
	 * 
	 * @param	propertyName	The name of the property we want to set
	 * @param	value			The value of the property
	 */
	public void setProperty(String propertyName, String value) {
		originalState.setState(propertyName, value);
	}
	
	/**
	 * Register a map of properties.
	 * <p>
	 * Invoking this method will cause all provided properties to be set onto
	 * the original state.
	 * 
	 * @param 	properties		A Map containing all properties we want to set
	 */
	public void register(Map<String, String> properties) {
		originalState.setStates(properties);
	}
	
// -- Parsing -----------------------------------------------------------------
	
	/**
	 * Parse the object for the given context.
	 * <p>
	 * A <code>StringParser</code> will be created to read the original 
	 * parameters and use them as input. The provided context will be passed
	 * to the parser so the properties inside this context can be injected into
	 * the parser's input.
	 * <p>
	 * The parser's output will result in a new 
	 * <code>MomentumObjectState</code> with the output of each property being 
	 * set as the current state.
	 *  
	 * @param context	The context for which we need to parse this object
	 */
	public void parse(Map<String, Object> context) {
		MomentumObjectState newState = new MomentumObjectState();
		
		// -- create the parameterized String
		StringParser parser = new StringParser(new ContextParameterHandler(context));

		for (String parameterName : state) {
			if (parameterName == null) continue;
			
			// -- create the parameterizedString
			String stateValue = parser.parse(state.getState(parameterName));
			
			// -- persist the parsed string inside the new state
			newState.setState(parameterName, stateValue);
		}
		
		state = newState;
	}
	
// -- State Reset -------------------------------------------------------------
	
	/**
	 * Reset the Object to it's original state
	 */
	public void reset() {
		state = originalState;
	}
	
// -- Getters -----------------------------------------------------------------	
	
	protected String getProperty(String propertyName) {
		return state.getState(propertyName);
	}
	
	protected Map<String, String> getStates() {
		return state.getStates();
	}

	protected MomentumObjectState getObjectState() {
		return state;
	}
	
	public String getInternalName() {
		return internalName;
	}
}
